SystemOrganization addCategory: #SlotsClassBuilderIntegration!

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 4/26/2013 17:40'!
addInstVarNamed: aString
	"Add the argument, aString, as one of the receiver's instance variables."
	^ PharoClassInstaller make: [ :builder |
		builder
			name: self name;
			superclass: self superclass;
			layoutClass: self layout class;
			slots: self layout allSlots, { aString asSlot };
			sharedVariables: self classVariablesString;
			sharedPools: self sharedPoolsString;
			category: self category.
			self flag: #todo. "set environment"
	]! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 4/26/2013 17:58'!
removeInstVarNamed: aString 
	"Remove the argument, aString, as one of the receiver's instance variables."

	| newInstVarString |
	(self instVarNames includes: aString)
		ifFalse: [self error: aString , ' is not one of my instance variables'].

	self flag: #fix.
	newInstVarString := ''.
	(self instVarNames copyWithout: aString) do: 
		[:varName | newInstVarString := newInstVarString , ' ' , varName].

	^ PharoClassInstaller make: [ :builder |
		builder
			name: self name;
			superclass: self superclass;
			layoutClass: self layout class;
			slots: newInstVarString asSlotCollection;
			sharedVariables: self classVariablesString;
			sharedPools: self sharedPoolsString;
			category: self category.
			self flag: #todo. "set environment"
	]! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:18'!
subclass: className instanceVariableNames: slotNamesString classVariableNames: sharedVariablesString poolDictionaries: sharedPoolsString category: categoryName
	"This is the standard initialization message for creating a new class as a 
	subclass of an existing class (the receiver)."

	^ self subclass: className 
		layoutClass: PointerLayout 
		slots: slotNamesString asSlotCollection 
		sharedVariableNames: sharedVariablesString 
		sharedPoolNames: sharedPoolsString 
		category: categoryName! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:16'!
subclass: className layoutClass: aLayoutClass slots: slotCollection sharedVariableNames: sharedVariablesString sharedPoolNames: sharedPoolsString category: categoryName
	"This is a message for creating a new class as a subclass of an existing class (the receiver)."

	^ PharoClassInstaller make: [ :builder |
		builder
			superclass: self;
			layoutClass: aLayoutClass;
			name: className;
			slots: slotCollection;
			sharedVariables: sharedVariablesString;
			sharedPools: sharedPoolsString;
			category: categoryName.
			
		"Class-side stuff shouldn't be lost. Probably this is responsibility of class builder."
		Smalltalk at: className ifPresent: [ :oldClass |
			builder 
				classSlots: (oldClass classSide instVarNames collect: [:n | n asSlot ]);
				classTraitComposition: oldClass classSide traitComposition.
				 ].
			]! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:19'!
variableByteSubclass: className instanceVariableNames: slotNamesString 
	classVariableNames: sharedVariablesString poolDictionaries: sharedPoolsString category: categoryName
	"This is the standard initialization message for creating a new class as a 
	subclass of an existing class (the receiver) in which the subclass is to 
	have indexable byte-sized nonpointer variables."

	^ self subclass: className 
		layoutClass: ByteLayout 
		slots: slotNamesString asSlotCollection 
		sharedVariableNames: sharedVariablesString 
		sharedPoolNames: sharedPoolsString 
		category: categoryName! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:19'!
variableSubclass: className instanceVariableNames: slotNamesString 
	classVariableNames: sharedVariablesString poolDictionaries: sharedPoolsString category: categoryName
	"This is the standard initialization message for creating a new class as a 
	subclass of an existing class (the receiver) in which the subclass is to 
	have indexable pointer variables."

	^ self subclass: className 
		layoutClass: VariableLayout 
		slots: slotNamesString asSlotCollection 
		sharedVariableNames: sharedVariablesString 
		sharedPoolNames: sharedPoolsString 
		category: categoryName! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:19'!
variableWordSubclass: className instanceVariableNames: slotNamesString 
	classVariableNames: sharedVariablesString poolDictionaries: sharedPoolsString category: categoryName
	"This is the standard initialization message for creating a new class as a 
	subclass of an existing class (the receiver) in which the subclass is to 
	have indexable word-sized nonpointer variables."

	^ self subclass: className 
		layoutClass: WordLayout 
		slots: slotNamesString asSlotCollection 
		sharedVariableNames: sharedVariablesString 
		sharedPoolNames: sharedPoolsString 
		category: categoryName! !

!Class methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 11:19'!
weakSubclass: className instanceVariableNames: slotNamesString 
	classVariableNames: sharedVariablesString poolDictionaries: sharedPoolsString category: categoryName
	"This is the standard initialization message for creating a new class as a subclass of an existing class (the receiver) in which the subclass is to have weak indexable pointer variables."

	^ self subclass: className 
		layoutClass: WeakLayout 
		slots: slotNamesString asSlotCollection 
		sharedVariableNames: sharedVariablesString 
		sharedPoolNames: sharedPoolsString 
		category: categoryName! !

!ClassTest methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 4/26/2013 17:54'!
testRemoveInstVarName

	| tutu |
	tutu := Smalltalk globals at: #TUTU.
	tutu addInstVarNamed: 'x'.
	self assert: tutu instVarNames = #('x').
	tutu removeInstVarNamed: 'x'.
	self assert: tutu instVarNames = #()! !

!ClassFactoryWithOrganization methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/11/2012 11:29'!
newClassNamed: aString subclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString category: category
	| newClass |
	newClass := aClass 
		subclass: aString 
		instanceVariableNames: ivNamesString 
		classVariableNames: classVarsString 
		poolDictionaries: '' 
		category: category asSymbol.
		
	self createdClasses add: newClass.
	^newClass! !

!ClassFactoryWithOrganization methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/11/2012 11:31'!
newSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString category: category

	^ self 
		newClassNamed: self newName 
		subclassOf: aClass 
		instanceVariableNames: ivNamesString 
		classVariableNames:  classVarsString 
		category: category! !

!MCClassDefinition methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/8/2012 00:02'!
createClass
	| superClass class composition typeOfClassToLayoutMap |
	superClass := Smalltalk globals at: superclassName.

	typeOfClassToLayoutMap := Dictionary new
		at: #compiledMethod put: CompiledMethodLayout;
		at: #bytes put: ByteLayout;
		at: #words put: WordLayout;
		at: #weak put: WeakLayout;
		at: #variable put: VariableLayout;
		at: #normal put: PointerLayout;
		yourself.
	class := PharoClassInstaller make: [ :builder |
		builder
			superclass: superClass;
			name: name;
			layoutClass: (typeOfClassToLayoutMap at: type);
			slots: self instanceVariablesString asSlotCollection;
			sharedVariables: self classVariablesString;
			sharedPools: self sharedPoolsString;
			category: category ].

	self traitComposition
		ifNotNil: [ composition := self class evaluatorClass evaluate: self traitComposition.
				  ((((composition isCollection and: [ ( composition includes: nil ) not])  
					or: [composition isKindOf: TraitComposition ])
						or: [composition isKindOf: Trait ])
							or: [composition isKindOf: TraitTransformation ])
					ifTrue: [ class setTraitComposition: composition asTraitComposition ] ].
	self classTraitComposition
		ifNotNil: [ composition := self class evaluatorClass evaluate: self classTraitComposition.
				  ((((composition isCollection and: [ ( composition includes: nil ) not])  
					or: [composition isKindOf: TraitComposition ])
						or: [composition isKindOf: Trait ])
							or: [composition isKindOf: TraitTransformation ])
					ifTrue: [ class class setTraitComposition: composition asTraitComposition ] ].
	^ class! !

!Metaclass methodsFor: '*SlotsClassBuilderIntegration' stamp: 'MartinDias 8/12/2012 10:11'!
instanceVariableNames: instVarString 
	"Declare additional named variables for my instance."

	PharoClassInstaller make: [ :builder |
		builder
			superclass: thisClass superclass;
			name: thisClass name;
			layoutClass: thisClass layout class;
			slots: (thisClass instVarNames collect: [:n | n asSlot ]);
			sharedVariables: thisClass classVariablesString;
			sharedPools: thisClass sharedPoolsString;
			traitComposition: thisClass traitComposition;
			category: thisClass category asString;
			classSlots: instVarString asSlotCollection
			]! !
